@@ -14,17 +14,26 @@ NULL
 #' @name Objective-class
 #'
 #' @family classes
+#'
+#' @export
 Objective <- R6::R6Class(
   "Objective",
   inherit = ConservationModifier,
   public = list(
 
+    #' @field has_targets `logical` value indicating if the objective
+    #' supports targets.
+    has_targets = new_waiver(),
+
+    #' @field has_weights `logical` value indicating if the objective
+    #' supports feature weights.
+    has_weights = new_waiver(),
+
     #' @description
-    #' Specify default values for weights.
-    #' @param x [problem()] object.
-    #' @return Invisible `TRUE`.
-    default_weights = function(x) {
-      return(1)
+    #' Specify default value for the feature weights.
+    #' @return A `numeric` value.
+    default_weights = function() {
+      1
     },
 
     #' @description
