@@ -17,17 +17,28 @@ NULL
 #'   See the Data format section for more information.
 #'
 #' @section Data format:
+#'
 #' The argument to `data` should be a `data.frame` with the following columns:
 #'
 #' \describe{
 #'
-#' \item{pu}{`integer` planning unit identifier.}
+#' \item{pu}{`integer` planning unit identifiers.
+#'   If `x` has `data.frame` planning units,
+#'   then these values must refer to values in the `id` column of the planning
+#'   unit data.
+#'   Alternatively, if `x` has [sf::st_sf()] or `matrix` planning units,
+#'   then these values must refer to the row numbers of the planning unit data.
+#'   Additionally, if `x` has `numeric` vector planning units,
+#'   then these values must refer to the element indices of the planning unit
+#'   data.
+#'   Finally, if `x` has [terra::rast()] planning units,
+#'   then these values must refer to cell indices.}
 #'
 #' \item{zone}{`character` names of zones. Note that this
 #'   argument is optional for arguments to `x` that contain a single
 #'   zone.}
 #'
-#' \item{status}{`numeric` values indicating how much
+#' \item{status}{`numeric` status values. These values indicate how much
 #'   of each planning unit should be allocated to each zone in the solution.
 #'   For example, the `numeric` values could be binary values (i.e., zero
 #'   or one) for problems containing binary-type decision variables
@@ -186,21 +197,25 @@ methods::setMethod("add_manual_locked_constraints",
 methods::setMethod("add_manual_locked_constraints",
   methods::signature("ConservationProblem", "tbl_df"),
   function(x, data) {
+    # assert valid arguments
     assert(
+      ## x
       is_conservation_problem(x),
+      ## data
       is.data.frame(data),
       inherits(data, "tbl_df"),
       nrow(data) > 0,
+      ### data$pu
       assertthat::has_name(data, "pu"),
       is.numeric(data$pu),
       all_finite(data$pu),
-      is_count_vector(data$pu),
-      max(data$pu) <= number_of_total_units(x),
-      min(data$pu) >= 0,
+      all_is_valid_total_unit_ids(x, data$pu),
+      ### data$status
       assertthat::has_name(data, "status"),
       is.numeric(data$status),
       all_finite(data$status)
     )
+    ## data$zone
     if (assertthat::has_name(data, "zone") || x$number_of_zones() > 1) {
       assert(
         assertthat::has_name(data, "zone"),
@@ -208,61 +223,156 @@ methods::setMethod("add_manual_locked_constraints",
         all_match_of(as.character(data$zone), zone_names(x))
       )
     }
-    # set attributes
-    if (x$number_of_zones() == 1) {
-      if (all(data$status == 1)) {
-        class_name <- "LockedInConstraint"
-        constraint_name <- "locked in constraints"
-      } else if (all(data$status == 0)) {
-         class_name <- "LockedOutConstraint"
-        constraint_name <- "locked out constraints"
-      } else {
-       class_name <- "LockedManualConstraint"
-         constraint_name <- "manual locked constraints"
-      }
-    } else {
-      class_name <- "LockedManualConstraint"
-      constraint_name <- "manual locked constraints"
-    }
+    # convert planning unit identifiers to indices
+    data$idx <- x$convert_total_unit_ids_to_indices(data$pu)
     # add constraints
-    x$add_constraint(
-      R6::R6Class(
-        class_name,
-        inherit = Constraint,
-        public = list(
-          name = constraint_name,
-          data = list(data = data),
-          repr = function(compact = TRUE) {
-            paste0(
-              self$name, " (", nrow(self$get_data("data")), " planning units)"
-            )
-          },
-          apply = function(x, y) {
-            # assert argument is valid
-            assert(
-              inherits(x, "OptimizationProblem"),
-              inherits(y, "ConservationProblem"),
-              .internal = TRUE
-            )
-            # get locked data
-            d <- self$get_data("data")
-            # convert zone names to indices
-            if (!assertthat::has_name(d, "zone"))
-              d$zone <- y$zone_names()[1]
-            d$zone <- match(as.character(d$zone), y$zone_names())
-            # convert from total units to planning units
-            units <- y$planning_unit_indices()
-            d$pu <- match(d$pu, units)
-            d <- d[!is.na(d$pu), , drop = FALSE]
-            # apply constraints
-            invisible(
-              rcpp_apply_locked_constraints(
-                x$ptr, c(d$pu), c(d$zone), d$status
-              )
-            )
-          }
-        )
-      )$new()
+    internal_add_manual_locked_constraints(
+      x,
+      data[!is.na(data$idx), , drop = FALSE],
+      lb = TRUE,
+      ub = TRUE
     )
   }
 )
+
+#' Internal function for add_manual_locked_constraints()
+#'
+#' This function is used to add manual locked constraints.
+#'
+#' @param x [problem()] object.
+#'
+#' @param data `data.frame` or [tibble::tibble()] object.
+#'   See the Data format section for more information.
+#'
+#' @param lb `logical` indicating if the status values should be used to
+#'   specify the lower bounds for the decision variables in the problem
+#'   formulation.
+#'   Defaults to `TRUE`.
+#'
+#' @param ub `logical` indicating if the status values should be used to
+#'   specify the upper bounds for the decision variables in the problem
+#'   formulation.
+#'   Defaults to `TRUE`.
+#'
+#' @section Data format:
+#' The argument to `data` should be a `data.frame` with the following columns:
+#'
+#' \describe{
+#'
+#' \item{idx}{`integer` total unit indices.
+#'  These values should refer to `x$total_unit_indices()`}.
+#'
+#' \item{zone}{`character` names of zones. Note that this
+#'   argument is optional for arguments to `x` that contain a single
+#'   zone.}
+#'
+#' \item{status}{`numeric` values indicating how much
+#'   of each planning unit should be allocated to each zone in the solution.
+#'   For example, the `numeric` values could be binary values (i.e., zero
+#'   or one) for problems containing binary-type decision variables
+#'   (using the [add_binary_decisions()] function). Alternatively,
+#'   the `numeric` values could be proportions (e.g., 0.5) for problems
+#'   containing proportion-type decision variables (using the
+#'   [add_proportion_decisions()]).}
+#'
+#'  }
+#'
+#' @inherit add_contiguity_constraints return
+#'
+#' @noRd
+internal_add_manual_locked_constraints <- function(
+  x, data, lb = TRUE, ub = TRUE, call = fn_caller_env()
+) {
+  assert(
+    ## x
+    is_conservation_problem(x),
+    is.data.frame(data),
+    ## data
+    inherits(data, "tbl_df"),
+    nrow(data) > 0,
+    ## data$idx
+    assertthat::has_name(data, "idx"),
+    is.numeric(data$idx),
+    all_finite(data$idx),
+    is_count_vector(data$idx),
+    ## data$status
+    assertthat::has_name(data, "status"),
+    is.numeric(data$status),
+    all_finite(data$status),
+    call = call
+  )
+  assert(
+    ## lb
+    assertthat::is.flag(lb),
+    assertthat::noNA(lb),
+    ## ub
+    assertthat::is.flag(ub),
+    assertthat::noNA(ub),
+    .internal = TRUE,
+    call = call
+  )
+  # set attributes
+  if (x$number_of_zones() == 1) {
+    if (all(data$status == 1)) {
+      class_name <- "LockedInConstraint"
+      constraint_name <- "locked in constraints"
+    } else if (all(data$status == 0)) {
+       class_name <- "LockedOutConstraint"
+      constraint_name <- "locked out constraints"
+    } else {
+     class_name <- "LockedManualConstraint"
+       constraint_name <- "manual locked constraints"
+    }
+  } else {
+    class_name <- "LockedManualConstraint"
+    constraint_name <- "manual locked constraints"
+  }
+  # add constraints
+  x$add_constraint(
+    R6::R6Class(
+      class_name,
+      inherit = Constraint,
+      public = list(
+        name = constraint_name,
+        data = list(data = data, lb = lb, ub = ub),
+        repr = function(compact = TRUE) {
+          paste0(
+            self$name, " (", nrow(self$get_data("data")), " planning units)"
+          )
+        },
+        apply = function(x, y) {
+          # assert argument is valid
+          assert(
+            inherits(x, "OptimizationProblem"),
+            inherits(y, "ConservationProblem"),
+            .internal = TRUE
+          )
+          # get locked data
+          d <- self$get_data("data")
+          lb <- self$get_data("lb")
+          ub <- self$get_data("ub")
+          # if locked data is missing a zone column,
+          # then this is because the problem has only one zone and the
+          # user did not specify this information.
+          # thus add in a column with the name of this single zone
+          if (!assertthat::has_name(d, "zone")) d$zone <- y$zone_names()[[1]]
+          # convert zone names to indices
+          d$zone <- match(as.character(d$zone), y$zone_names())
+          # convert total unit indices to indices that describe their
+          # relative position in the MILP formulation
+          i <- match(d$idx, y$planning_unit_indices())
+          # note that any total units which are not planning units --
+          # because they have NA cost values -- will have NA values in i.
+          # thus we need to exclude these values when applying the constraints
+          keep <- !is.na(i)
+          # apply constraints
+          invisible(
+            rcpp_apply_locked_constraints(
+              x$ptr, i[keep], d$zone[keep], d$status[keep], lb, ub
+            )
+          )
+        }
+      )
+    )$new()
+  )
+}
