@@ -1,9 +1,9 @@
-#' @include internal.R ConservationProblem-class.R zones.R tbl_df.R
+#' @include internal.R ConservationProblem-class.R zones.R tbl_df.R target_optimization_format.R
 NULL
 
 #' Add manual targets
 #'
-#' Set targets for a conservation planning problem by manually
+#' Add targets to a conservation planning problem by manually
 #' specifying all the required information for each target. This function
 #' is useful because it can be used to customize all aspects of a target. For
 #' most cases, targets can be specified using the
@@ -21,6 +21,8 @@ NULL
 #'
 #' @inherit add_absolute_targets details
 #'
+#' @inheritSection add_auto_targets Target setting
+#'
 #' @section Targets format:
 #'
 #' The `targets` argument should be a `data.frame` with the following
@@ -56,6 +58,11 @@ NULL
 #' @seealso
 #' See [targets] for an overview of all functions for adding targets.
 #'
+#' @references
+#' Carwardine J, Klein CJ, Wilson KA, Pressey RL, Possingham HP (2009) Hitting
+#' the target and missing the point: target‐based conservation planning in
+#' context. *Conservation Letters*, 2: 4--11.
+#'
 #' @family targets
 #'
 #' @examples
@@ -294,9 +301,24 @@ methods::setMethod(
       assert(
         assertthat::has_name(targets, "zone"),
         is_inherits(targets$zone, c("character", "factor", "list")),
-        is_inherits(unlist(targets$zone), c("character", "factor")),
         all_match_of(unlist(targets$zone), zone_names(x))
       )
+      if (is.list(targets$zone)) {
+        assert(
+          all_elements_inherit(targets$zone, c("character", "factor")),
+          msg = c(
+            "!" = paste0(
+              "{.arg targets$zone} must contain elements are that a ",
+              "{.cls character} or {.cls factor} value."
+            ),
+            "i" = paste0(
+              "If {.arg targets$zone} is a {.cls list} column, ",
+              "then each element of this {.cls list} column must contain a ",
+              "{.cls character} or {.cls factor} value or vector."
+            )
+          )
+        )
+      }
     }
     if (assertthat::has_name(targets, "sense")) {
       assert(
@@ -322,15 +344,31 @@ methods::setMethod(
     ) {
       verify(all_positive(targets$target))
     }
+    # if features have user-defined area units, then throw warning indicating
+    # that these targets do not consider the spatial units
+    verify(
+      inherits(x$data$cost, c("SpatRaster", "Raster")) ||
+        all(is.na(x$feature_units())),
+      msg = c(
+      "!" = "{.arg x} has spatial units defined for the features.",
+        "i" = paste(
+          "This function for adding targets does not account for spatial units."
+        ),
+        "i" = paste(
+          "See {.fn add_auto_targets} or {.fn add_group_targets} to",
+          "add targets that account for spatial units."
+        )
+      ),
+      call = NULL
+    )
     # add targets to problem
     x$add_targets(
       R6::R6Class(
         "ManualTargets",
         inherit = Target,
         public = list(
-          name = "targets",
+          name = "manual targets",
           data = list(targets = targets),
-          internal = list(abundances = x$feature_abundances_in_total_units()),
           repr = function(compact = TRUE) {
             d <- self$get_data("targets")
             if (all(as.character(d$type) == "relative")) {
@@ -340,50 +378,25 @@ methods::setMethod(
             } else {
               type <- "mixed"
             }
-            cli::format_inline(
-              "{type} targets (between {.val {range(d$target)}})"
-            )
+            if (identical(length(unique(d$target)), 1L)) {
+              out <-
+                "{type} targets (all equal to {repr.numeric(d$target[[1]])})"
+            } else {
+              out <-
+                "{type} targets (between {repr.numeric(range(d$target))})"
+            }
+            cli::format_inline(out)
           },
-          output = function() {
-            # get data
-            targets <- self$get_data("targets")
-            abundances <- self$get_internal("abundances")
-            # add zone column if missing
-            if (!assertthat::has_name(targets, "zone")) {
-              targets$zone <- colnames(abundances)[[1]]
-            }
-            # convert zone column to list of characters if needed
-            if (!inherits(targets$zone, "list")) {
-              targets$zone <- as.list(targets$zone)
-            }
-            # add sense column if missing
-            if (!assertthat::has_name(targets, "sense")) {
-              targets$sense <- ">="
-            }
-            targets$sense <- as.character(targets$sense)
-            # convert feature names to indices
-            targets$feature <- match(
-              as.character(targets$feature), rownames(abundances)
+          output = function(x) {
+            # assert x is a conservation problem
+            assert(
+              is_conservation_problem(x),
+              .internal = TRUE
             )
-            # convert zone names to indices
-            for (i in seq_len(nrow(targets))) {
-              targets$zone[[i]] <- match(
-                targets$zone[[i]], colnames(abundances)
-              )
-            }
-            # add compute relative targets as absolute targets and assign
-            # zone ids
-            targets$value <- as.numeric(targets$target)
-            relative_rows <- which(targets$type == "relative")
-            for (i in seq_along(relative_rows)) {
-              zone_id <- targets$zone[[relative_rows[[i]]]]
-              feature_id <- targets$feature[[relative_rows[[i]]]]
-              abund_mtx <- as.matrix(data.frame(feature_id, zone_id))
-              targets$value[relative_rows[i]] <-
-                sum(abundances[abund_mtx]) * targets$target[relative_rows[i]]
-             }
-            # return tibble
-            targets[, c("feature", "zone", "sense", "value")]
+            # get targets
+            targets <- self$get_data("targets")
+            # get targets for optimization
+            target_optimization_format(x, targets)
           }
         )
       )$new()
