@@ -45,7 +45,7 @@ NULL
 #' Here, \eqn{x_i}{xi} is the [decisions] variable (e.g.,
 #' specifying whether planning unit \eqn{i}{i} has been selected (1) or not
 #' (0)), \eqn{r_{ij}}{rij} is the amount of feature \eqn{j}{j} in planning
-#' unit \eqn{i}{i}, \eqn{A_j}{Aj} is the amount of feature \eqn{j}{j}
+#' unit \eqn{i}{i}, \eqn{a_j}{aj} is the amount of feature \eqn{j}{j}
 #' represented in in the solution, and \eqn{w_j}{wj} is the weight for
 #' feature \eqn{j}{j} (defaults to 1 for all features; see
 #' [add_feature_weights()]
@@ -140,11 +140,14 @@ add_max_utility_objective <- function(x, budget) {
       inherit = Objective,
       public = list(
         name = "maximum utility objective",
+        has_weights = TRUE,
+        has_targets = FALSE,
         data = list(budget = budget),
-        apply = function(x, y) {
+        apply = function(x, y, weights) {
           assert(
             inherits(x, "OptimizationProblem"),
             inherits(y, "ConservationProblem"),
+            is.numeric(weights),
             .internal = TRUE
           )
           invisible(
@@ -153,7 +156,8 @@ add_max_utility_objective <- function(x, budget) {
               unname(y$feature_positive_abundances_in_planning_units()),
               y$has_negative_feature_data(),
               y$planning_unit_costs(),
-              self$get_data("budget")
+              self$get_data("budget"),
+              weights
             )
           )
         }
