@@ -211,7 +211,8 @@ add_rsymphony_solver <- function(x, gap = 0.1,
             objective = x$objval,
             status = as.character(x$status),
             runtime = rt[[3]],
-            gap = NA_real_
+            gap = NA_real_,
+            objbound = NA_real_
           )
         }
       )
