@@ -4,7 +4,7 @@ NULL
 #' Binary stack
 #'
 #' Convert a single-layer [terra::rast()] object that contains integer values
-#' into a multi-layer [terra::rast()] object with pixel values denote the
+#' into a multi-layer [terra::rast()] object with cell values denote the
 #' presence/absence of a given integer value. This is methodology is also known
 #' as "one-hot encoding".
 #'
@@ -28,7 +28,9 @@ NULL
 #'
 #' @return A [terra::rast()] object.
 #'
-#' @seealso [category_layer()], [terra::segregate()].
+#' @seealso
+#' The [category_layer()] function performs the reverse of this function.
+#' Also the [terra::segregate()] function provides similar functionality.
 #'
 #' @examples
 #' # create raster with categorical values
@@ -76,14 +78,14 @@ binary_stack.SpatRaster <- function(x, keep_all = TRUE) {
   # create segregated raster
   y <- terra::segregate(
     x,
-    classes = seq_len(round(terra::global(x, "max", na.rm = TRUE)[[1]])),
+    classes = seq_len(round(terra::minmax(x, compute = TRUE)[2, 1])),
     keep = FALSE,
     round = TRUE
   )
 
   # if needed, remove empty layers
   if (!isTRUE(keep_all)) {
-    idx <- which(terra::global(y, "max", na.rm = TRUE)[[1]] > 0)
+    idx <- which(terra::minmax(y, compute = TRUE)[2, ] > 0)
     y <- y[[idx]]
   }
 
