@@ -28,6 +28,9 @@ NULL
 #' \item{`add_corridor_constraints()`}{replaced by the
 #'   [add_feature_contiguity_constraints()] function.}
 #'
+#' \item{`add_loglinear_targets()`}{replaced by the
+#'   [spec_interp_absolute_targets()] function.}
+#'
 #' \item{`set_number_of_threads()`}{
 #'   no longer needed due to improved data extraction methods.}
 #'
@@ -198,6 +201,15 @@ predefined_optimization_problem <- function(...) {
   )
 }
 
+#' @rdname prioritizr-deprecated
+#' @export
+add_loglinear_targets <- function(...) {
+  cli_defunct(
+    old = "add_loglinear_targets",
+    new = "spec_interp_absolute_targets"
+  )
+}
+
 raster_pkg_deprecation_notice <- c(
   "Support for {.pkg raster} package will be deprecated.",
   "i" = "Use {.fn terra::rast} to convert data for future compatibility."
