@@ -428,8 +428,7 @@ methods::setMethod(
     )
     ## values in the solution
     assert(
-      min(terra::global(solution, "min", na.rm = TRUE)[[1]]) >= 0,
-      max(terra::global(solution, "max", na.rm = TRUE)[[1]]) <= 1,
+      all_proportion(solution),
       call = call,
       msg = paste(
         "{.arg solution} must only contain",
@@ -539,7 +538,7 @@ solution_format_documentation <- function(x) {
 }
 
 \\item{`x` has [terra::rast()] planning units}{The argument to `", x , "`
-  be a [terra::rast()] object where different grid cells (pixels) correspond
+  be a [terra::rast()] object where different cells correspond
   to different planning units and layers correspond to
   a different management zones. It should have the same dimensionality
   (rows, columns, layers), resolution, extent, and coordinate reference
