@@ -3,9 +3,13 @@
 
 // [[Rcpp::export]]
 bool rcpp_apply_max_phylo_objective(SEXP x,
-  const Rcpp::List targets_list, const Rcpp::NumericMatrix costs,
-  const Rcpp::NumericVector budget, const arma::sp_mat branch_matrix,
-  const Rcpp::NumericVector branch_lengths) {
+  const Rcpp::List targets_list,
+  const Rcpp::NumericMatrix costs,
+  const Rcpp::NumericVector budget,
+  const arma::sp_mat branch_matrix,
+  const Rcpp::NumericVector branch_lengths,
+  const Rcpp::NumericVector weights
+) {
   // initialize
   Rcpp::XPtr<OPTIMIZATIONPROBLEM> ptr = Rcpp::as<Rcpp::XPtr<OPTIMIZATIONPROBLEM>>(x);
   std::size_t A_extra_ncol;
@@ -62,9 +66,9 @@ bool rcpp_apply_max_phylo_objective(SEXP x,
   if (!ptr->_compressed_formulation)
     for (std::size_t i = 0; i < A_extra_ncol; ++i)
        ptr->_obj.push_back(0.0);
-  // add in default feature weights (i.e. zero)
+  // add in feature weights
   for (std::size_t i = 0; i < n_targets; ++i)
-    ptr->_obj.push_back(0.0);
+    ptr->_obj.push_back(weights[i]);
   // add in branch lengths into objective function
   for (auto it = branch_lengths.begin(); it!=branch_lengths.end(); ++it)
     ptr->_obj.push_back(*it);
