@@ -1,5 +1,6 @@
 test_that("binary decisions", {
   skip_on_cran()
+  skip_on_os("windows")
   skip_if_not_installed("highs")
   # load data
   sim_pu_raster <- get_sim_pu_raster()
@@ -30,6 +31,7 @@ test_that("binary decisions", {
 
 test_that("proportion decisions", {
   skip_on_cran()
+  skip_on_os("windows")
   skip_if_not_installed("highs")
   # load data
   sim_pu_raster <- get_sim_pu_raster()
@@ -56,6 +58,7 @@ test_that("proportion decisions", {
 
 test_that("proportion decisions (floating point)", {
   skip_on_cran()
+  skip_on_os("windows")
   skip_if_not_installed("highs")
   skip_if_not_installed("prioritizrdata", minimum_version = "0.3.0")
   # import data
@@ -108,6 +111,8 @@ test_that("variable bounds methods", {
 })
 
 test_that("threads patched", {
+  skip_on_cran()
+  skip_on_os("windows")
   skip_if_not_installed("highs")
   # import data
   sim_pu_raster <- get_sim_pu_raster()
@@ -137,6 +142,8 @@ test_that("threads patched", {
 })
 
 test_that("small values in constraint matrix", {
+  skip_on_cran()
+  skip_on_os("windows")
   skip_if_not_installed("highs")
   # import data
   sim_pu_raster <- get_sim_pu_raster()
@@ -158,6 +165,8 @@ test_that("small values in constraint matrix", {
 })
 
 test_that("small values in objective function", {
+  skip_on_cran()
+  skip_on_os("windows")
   skip_if_not_installed("highs")
   # import data
   sim_pu_raster <- get_sim_pu_raster()
@@ -180,6 +189,7 @@ test_that("small values in objective function", {
 
 test_that("mix of binary and continuous variables", {
   skip_on_cran()
+  skip_on_os("windows")
   skip_if_not_installed("highs")
   # import data
   sim_pu_raster <- get_sim_pu_raster()
@@ -203,6 +213,7 @@ test_that("mix of binary and continuous variables", {
 
 test_that("correct solution (simple)", {
   skip_on_cran()
+  skip_on_os("windows")
   skip_if_not_installed("highs")
   # create data
   cost <- terra::rast(matrix(c(1, 2, 2, NA), ncol = 4))
@@ -231,6 +242,7 @@ test_that("correct solution (simple)", {
 
 test_that("correct solution (complex)", {
   skip_on_cran()
+  skip_on_os("windows")
   skip_if_not_installed("highs")
   # create data
   cost <- terra::rast(matrix(c(1000, 100, 200, 300, NA), nrow = 1))
@@ -263,6 +275,7 @@ test_that("correct solution (complex)", {
 
 test_that("solver information (single solution)", {
   skip_on_cran()
+  skip_on_os("windows")
   skip_if_not_installed("highs")
   # load data
   sim_pu_raster <- get_sim_pu_raster()
@@ -285,10 +298,13 @@ test_that("solver information (single solution)", {
   expect_length(attr(s, "status"), 1)
   expect_true(is.numeric(attr(s, "gap")))
   expect_length(attr(s, "gap"), 1)
+  expect_true(is.numeric(attr(s, "objbound")))
+  expect_length(attr(s, "objbound"), 1)
 })
 
 test_that("solver information (multiple solutions)", {
   skip_on_cran()
+  skip_on_os("windows")
   skip_if_not_installed("highs")
   # load data
   sim_pu_raster <- get_sim_pu_raster()
@@ -312,6 +328,8 @@ test_that("solver information (multiple solutions)", {
   expect_length(attr(s, "status"), 3)
   expect_true(is.numeric(attr(s, "gap")))
   expect_length(attr(s, "gap"), 3)
+  expect_true(is.numeric(attr(s, "objbound")))
+  expect_length(attr(s, "objbound"), 3)
 })
 
 test_that("set_start_solution", {
@@ -342,9 +360,9 @@ test_that("set_start_solution", {
   # force calculations
   p$solver$calculate(compile(p))
   # tests
-  expect_error(
+  expect_warning(
     p$solver$set_start_solution(c(1, 2, 3)),
-    "No defined"
+    "starting"
   )
 })
 
@@ -474,3 +492,29 @@ test_that("set_variable_ub", {
     c(1, 0, 1, 0)
   )
 })
+
+test_that("control", {
+  skip_on_cran()
+  skip_on_os("windows")
+  skip_if_not_installed("highs")
+  # load data
+  sim_pu_raster <- get_sim_pu_raster()
+  sim_features <- get_sim_features()
+  # create problems
+  p1 <-
+    problem(sim_pu_raster, sim_features) %>%
+    add_min_set_objective() %>%
+    add_relative_targets(0.1) %>%
+    add_binary_decisions() %>%
+    add_highs_solver(
+      gap = 100, verbose = FALSE,
+      control = list(solver = "simplex", ranging = "on")
+  )
+  # solve problems
+  s1 <- solve(p1)
+  # tests
+  expect_inherits(s1, "SpatRaster")
+  expect_equal(terra::nlyr(s1), 1)
+  expect_true(all_binary(s1))
+  expect_true(is_comparable_raster(sim_pu_raster, s1))
+})
