@@ -13,7 +13,7 @@ test_that("add_manual_targets (default, single zone)", {
       )
     )
   # calculate absolute targets
-  targets <- p$targets$output()
+  targets <- p$targets$output(p)
   # tests
   print(p)
   expect_inherits(targets, "tbl_df")
@@ -43,7 +43,7 @@ test_that("add_manual_targets (mixed, single zone)", {
       )
     )
   # calculate absolute targets
-  targets <- p$targets$output()
+  targets <- p$targets$output(p)
   # run tests
   print(p)
   expect_inherits(targets, "tbl_df")
@@ -78,7 +78,7 @@ test_that("add_manual_targets (explicit, single zone)", {
       )
     )
   # calculate absolute targets
-  targets <- p$targets$output()
+  targets <- p$targets$output(p)
   # tests
   expect_inherits(targets, "tbl_df")
   expect_true(all(names(targets) == c("feature", "zone", "sense", "value")))
@@ -108,7 +108,7 @@ test_that("add_manual_targets (default, multiple zones)", {
       )
     )
   # calculate absolute targets
-  targets <- p$targets$output()
+  targets <- p$targets$output(p)
   # tests
   expect_inherits(targets, "tbl_df")
   expect_true(all(names(targets) == c("feature", "zone", "sense", "value")))
@@ -139,7 +139,7 @@ test_that("add_manual_targets (explicit, multiple zones, negative)", {
       )
     )
   # calculate absolute targets
-  targets <- p$targets$output()
+  targets <- p$targets$output(p)
   # tests
   expect_inherits(targets, "tbl_df")
   expect_true(all(names(targets) == c("feature", "zone", "sense", "value")))
@@ -173,7 +173,7 @@ test_that("add_manual_targets (explicit, multiple zones)", {
     "negative"
   )
   # calculate absolute targets
-  targets <- p$targets$output()
+  targets <- p$targets$output(p)
   # tests
   expect_inherits(targets, "tbl_df")
   expect_true(all(names(targets) == c("feature", "zone", "sense", "value")))
@@ -285,4 +285,47 @@ test_that("add_manual_targets (invalid input)", {
       )
     )
   )
+  expect_tidy_error(
+    add_manual_targets(
+      p,
+      tibble::tibble(
+        feature = feature_names(sim_zones_features)[[1]],
+        zone = list(list(zone_names(sim_zones_features))),
+        sense = ">=",
+        type = "absolute",
+        target = 1
+      )
+    )
+  )
+})
+
+test_that("warnings", {
+  # load data
+  sim_pu_polygons <- get_sim_pu_polygons()
+  # prepare data
+  sim_pu_polygons$spp_1 <- runif(nrow(sim_pu_polygons))
+  sim_pu_polygons$spp_2 <- runif(nrow(sim_pu_polygons))
+  sim_pu_polygons$spp_3 <- runif(nrow(sim_pu_polygons))
+  # create problem
+  p <-
+    problem(
+      sim_pu_polygons,
+      c("spp_1", "spp_2", "spp_3"),
+      cost_column = "cost",
+      feature_units = "km2"
+    ) %>%
+    add_min_set_objective()
+  # run tests
+  expect_warning(
+    add_manual_targets(
+      p,
+      data.frame(
+        feature = "spp_1",
+        sense = ">=",
+        type = "absolute",
+        target = 1
+      )
+    ),
+    "spatial units"
+  )
 })
