@@ -223,6 +223,8 @@ test_that("solver information (single solution)", {
   expect_length(attr(s, "status"), 1)
   expect_true(is.numeric(attr(s, "gap")))
   expect_length(attr(s, "gap"), 1)
+  expect_true(is.numeric(attr(s, "objbound")))
+  expect_length(attr(s, "objbound"), 1)
 })
 
 test_that("solver information (multiple solutions)", {
@@ -250,6 +252,8 @@ test_that("solver information (multiple solutions)", {
   expect_length(attr(s, "status"), 3)
   expect_true(is.numeric(attr(s, "gap")))
   expect_length(attr(s, "gap"), 3)
+  expect_true(is.numeric(attr(s, "objbound")))
+  expect_length(attr(s, "objbound"), 3)
 })
 
 test_that("set_start_solution", {
@@ -280,9 +284,9 @@ test_that("set_start_solution", {
   # force calculations
   p$solver$calculate(compile(p))
   # tests
-  expect_error(
+  expect_warning(
     p$solver$set_start_solution(c(1, 2, 3)),
-    "No defined"
+    "starting"
   )
 })
 
