@@ -1,4 +1,4 @@
-test_that("SpatRaster", {
+test_that("SpatRaster (binary)", {
   # create data
   x <- c(
     terra::rast(matrix(c(1, 0, 0, 0, NA, 0), byrow = TRUE, nrow = 3)),
@@ -11,10 +11,26 @@ test_that("SpatRaster", {
   expect_inherits(y, "SpatRaster")
   expect_equal(terra::nlyr(y), 1)
   expect_true(is_comparable_raster(x, y))
-  expect_equal(c(terra::values(y)), c(1, 2, 3, 0, NA, 3))
+  expect_equal(c(terra::values(y)), c(1, 2, 3, 0, NaN, 3))
 })
 
-test_that("Raster", {
+test_that("SpatRaster (continuous)", {
+  # create data
+  x <- c(
+    terra::rast(matrix(c(0.9, 0.1, 0, 0, NA, 0), byrow = TRUE, nrow = 3)),
+    terra::rast(matrix(c(0.3, 0.3, 0, 0, NA, 0), byrow = TRUE, nrow = 3)),
+    terra::rast(matrix(c(0,   0,   1, 0, NA, 1), byrow = TRUE, nrow = 3))
+  )
+  # convert to binary stack
+  y <- category_layer(x)
+  # tests
+  expect_inherits(y, "SpatRaster")
+  expect_equal(terra::nlyr(y), 1)
+  expect_true(is_comparable_raster(x, y))
+  expect_equal(c(terra::values(y)), c(1, 2, 3, 0, NaN, 3))
+})
+
+test_that("Raster (binary)", {
   # create data
   x <- c(
     terra::rast(matrix(c(1, 0, 0, 1, NA, 0), byrow = TRUE, nrow = 3)),
