@@ -26,7 +26,7 @@ test_that("single zone (x = ConservationProblem)", {
   d$boundary <- 1
   d$boundary[d$id1 == d$id2] <- 0.5
   p <- problem(
-    x = runif(5),
+    x = matrix(runif(5), ncol = 1),
     features = data.frame(id = seq_len(2), name = letters[seq_len(2)]),
     rij_matrix = matrix(1, ncol = 5, nrow = 2)
   )
