@@ -2,6 +2,10 @@ test_that("x = RasterLayer, features = RasterStack", {
   # import data
   sim_pu_raster <- get_sim_pu_raster()
   sim_features <- get_sim_features()
+  # define spatial properties
+  terra::crs(sim_pu_raster) <- terra::crs("epsg:3857")
+  terra::crs(sim_features) <- terra::crs(sim_pu_raster)
+  unit_factor <- prod(terra::res(sim_features)) / (1000 * 1000)
   # create problem
   expect_warning(
     x <- problem(raster::raster(sim_pu_raster), raster::stack(sim_features)),
@@ -10,9 +14,13 @@ test_that("x = RasterLayer, features = RasterStack", {
   # verify that object can be printed
   suppressMessages(print(x))
   suppressMessages(x)
-  # tests for integer fields
+  # test for logical fields
+  expect_true(x$is_ids_equivalent_to_indices())
+  # tests for character fields
+  expect_equal(x$planning_unit_class(), "RasterLayer")
   expect_equal(x$feature_names(), names(sim_features))
   expect_equal(x$zone_names(), names(sim_pu_raster))
+  # tests for integer fields
   expect_equal(x$number_of_features(), terra::nlyr(sim_features))
   expect_equal(
     x$number_of_planning_units(),
@@ -25,6 +33,7 @@ test_that("x = RasterLayer, features = RasterStack", {
     terra::cells(is.na(sim_pu_raster), 0)[[1]]
   )
   expect_equal(problem(sim_pu_raster, sim_pu_raster)$number_of_features(), 1L)
+  expect_error(x$total_unit_ids())
   # tests for planning_unit_costs field
   expect_equal(
     x$planning_unit_costs(),
@@ -64,6 +73,21 @@ test_that("x = RasterLayer, features = RasterStack", {
     rownames(x$feature_abundances_in_total_units()),
     x$feature_names()
   )
+  # tests for feature_abundances_km2_in_total_units field
+  expect_equal(
+    x$feature_abundances_km2_in_total_units(),
+    matrix(terra::global(sim_features, "sum", na.rm = TRUE)[[1]], ncol = 1) *
+    unit_factor,
+    ignore_attr = TRUE
+  )
+  expect_equal(
+    colnames(x$feature_abundances_km2_in_total_units()),
+    x$zone_names()
+  )
+  expect_equal(
+    rownames(x$feature_abundances_km2_in_total_units()),
+    x$feature_names()
+  )
   # tests for rij_matrix field
   expect_equal(
     x$data$rij_matrix,
@@ -72,6 +96,11 @@ test_that("x = RasterLayer, features = RasterStack", {
   )
   expect_equal(names(x$data$rij_matrix), x$zone_names())
   expect_equal(rownames(x$data$rij_matrix[[1]]), x$feature_names())
+  # test for converting total unit ids to indices
+  expect_equal(
+    x$convert_total_unit_ids_to_indices(c(seq_len(3), 1e5, 4)),
+    c(seq_len(3), 1e5, 4)
+  )
   # test that calling targets before they have been initialized throws error
   expect_error(x$feature_targets())
 })
@@ -81,6 +110,13 @@ test_that("x = RasterStack, features = ZonesRaster", {
   sim_zones_pu_raster <- get_sim_zones_pu_raster()
   sim_zones_features <- get_sim_zones_features()
   sim_units_mask <- max(!is.na(sim_zones_pu_raster))
+  # define spatial properties
+  terra::crs(sim_zones_pu_raster) <- terra::crs("epsg:3857")
+  sim_zones_features <- set_zones_crs(
+    sim_zones_features, terra::crs("epsg:3857")
+  )
+  terra::crs(sim_units_mask) <- terra::crs(sim_zones_pu_raster)
+  unit_factor <- prod(terra::res(sim_zones_features[[1]])) / (1000 * 1000)
   # create problem
   expect_warning(
     x <- problem(
@@ -91,9 +127,13 @@ test_that("x = RasterStack, features = ZonesRaster", {
   # verify that object can be printed
   suppressMessages(print(x))
   suppressMessages(x)
-  # tests for integer fields
+  # test for logical fields
+  expect_true(x$is_ids_equivalent_to_indices())
+  # tests for character fields
+  expect_equal(x$planning_unit_class(), "RasterStack")
   expect_equal(x$feature_names(), feature_names(sim_zones_features))
   expect_equal(x$zone_names(), zone_names(sim_zones_features))
+  # tests for integer fields
   expect_equal(x$number_of_features(), number_of_features(sim_zones_features))
   expect_equal(x$number_of_zones(), number_of_zones(sim_zones_features))
   expect_equal(
@@ -115,6 +155,7 @@ test_that("x = RasterStack, features = ZonesRaster", {
     colnames(x$planning_unit_costs()),
     zone_names(sim_zones_features)
   )
+  expect_error(x$total_unit_ids())
   # tests for feature_abundances_in_planning_units field
   expect_equal(
     x$feature_abundances_in_planning_units(),
@@ -150,6 +191,23 @@ test_that("x = RasterStack, features = ZonesRaster", {
     rownames(x$feature_abundances_in_total_units()),
     feature_names(sim_zones_features)
   )
+  # tests for feature_abundances_km2_in_total_units field
+  expect_equal(
+    x$feature_abundances_km2_in_total_units(),
+    sapply(seq_len(terra::nlyr(sim_zones_pu_raster)), function(i) {
+      terra::global(sim_zones_features[[i]], "sum", na.rm = TRUE)[[1]] *
+      unit_factor
+    }),
+    ignore_attr = TRUE
+  )
+  expect_equal(
+    colnames(x$feature_abundances_km2_in_total_units()),
+    zone_names(sim_zones_features)
+  )
+  expect_equal(
+    rownames(x$feature_abundances_km2_in_total_units()),
+    feature_names(sim_zones_features)
+  )
   # tests for rij_matrix field
   expect_equal(
     x$data$rij_matrix,
@@ -175,6 +233,11 @@ test_that("x = RasterStack, features = ZonesRaster", {
     ),
     ignore_attr = TRUE
   )
+  # test for converting total unit ids to indices
+  expect_equal(
+    x$convert_total_unit_ids_to_indices(c(seq_len(3), 1e5, 4)),
+    c(seq_len(3), 1e5, 4)
+  )
   # test that calling targets before they have been initialized throws error
   expect_error(x$feature_targets())
 })
@@ -183,6 +246,12 @@ test_that("x = Spatial, features = RasterStack", {
   # import data
   sim_pu_polygons <- get_sim_pu_polygons()
   sim_features <- get_sim_features()
+  # define spatial properties
+  terra::crs(sim_features) <- terra::crs("epsg:3857")
+  suppressWarnings(
+    sf::st_crs(sim_pu_polygons) <- sf::st_crs("epsg:3857")
+  )
+  unit_factor <- prod(terra::res(sim_features[[1]])) / (1000 * 1000)
   # update data
   sim_pu_polygons$cost[1:5] <- NA
   # create problem
@@ -197,9 +266,13 @@ test_that("x = Spatial, features = RasterStack", {
   # verify that object can be printed
   suppressMessages(print(x))
   suppressMessages(x)
-  # tests for integer fields
-  expect_equal(x$feature_names(), names(sim_features))
+  # test for logical fields
+  expect_true(x$is_ids_equivalent_to_indices())
+  # tests for character fields
+  expect_equal(x$planning_unit_class(), "SpatialPolygonsDataFrame")
   expect_equal(x$zone_names(), "cost")
+  expect_equal(x$feature_names(), names(sim_features))
+  # tests for integer fields
   expect_equal(x$number_of_features(), terra::nlyr(sim_features))
   expect_equal(x$number_of_planning_units(), sum(!is.na(sim_pu_polygons$cost)))
   expect_equal(x$planning_unit_indices(), which(!is.na(sim_pu_polygons$cost)))
@@ -225,6 +298,7 @@ test_that("x = Spatial, features = RasterStack", {
     rownames(x$feature_abundances_in_planning_units()),
     names(sim_features)
   )
+  expect_error(x$total_unit_ids())
   # tests for feature_abundances_in_total_units field
   expect_equal(
     x$feature_abundances_in_total_units(),
@@ -247,6 +321,28 @@ test_that("x = Spatial, features = RasterStack", {
     rownames(x$feature_abundances_in_total_units()),
     names(sim_features)
   )
+  # tests for feature_abundances_km2_in_total_units field
+  expect_equal(
+    x$feature_abundances_km2_in_total_units(),
+    matrix(
+      colSums(
+        terra::extract(
+          sim_features, terra::vect(sim_pu_polygons), "sum", na.rm = TRUE,
+          ID = FALSE
+        ) * unit_factor
+      ),
+      ncol = 1
+    ),
+    ignore_attr = TRUE
+  )
+  expect_equal(
+    colnames(x$feature_abundances_km2_in_total_units()),
+    "cost"
+  )
+  expect_equal(
+    rownames(x$feature_abundances_km2_in_total_units()),
+    names(sim_features)
+  )
   # tests for rij_matrix field
   expect_equal(
     x$data$rij_matrix,
@@ -260,6 +356,11 @@ test_that("x = Spatial, features = RasterStack", {
   )
   expect_equal(names(x$data$rij_matrix), "cost")
   expect_equal(rownames(x$data$rij_matrix[[1]]), names(sim_features))
+  # test for converting total unit ids to indices
+  expect_equal(
+    x$convert_total_unit_ids_to_indices(c(seq_len(3), 1e5, 4)),
+    c(seq_len(3), 1e5, 4)
+  )
   # test that calling targets before they have been initialized throws error
   expect_error(x$feature_targets())
 })
@@ -271,6 +372,14 @@ test_that("x = Spatial, features = ZonesRaster", {
   # update data
   sim_zones_pu_polygons[5, paste0("cost_", 1:3)] <- NA
   sim_zones_pu_polygons[4, "cost_1"] <- NA
+  # define spatial properties
+  suppressWarnings(
+    sf::st_crs(sim_zones_pu_polygons) <- sf::st_crs(3857)
+  )
+  sim_zones_features <- set_zones_crs(
+    sim_zones_features, terra::crs("epsg:3857")
+  )
+  unit_factor <- prod(terra::res(sim_zones_features[[1]])) / (1000 * 1000)
   # create problem
   suppressWarnings(
     expect_warning(
@@ -285,9 +394,13 @@ test_that("x = Spatial, features = ZonesRaster", {
   # verify that object can be printed
   suppressMessages(print(x))
   suppressMessages(x)
-  # tests for integer fields
+  # test for logical fields
+  expect_true(x$is_ids_equivalent_to_indices())
+  # tests for character fields
+  expect_equal(x$planning_unit_class(), "SpatialPolygonsDataFrame")
   expect_equal(x$feature_names(), feature_names(sim_zones_features))
   expect_equal(x$zone_names(), zone_names(sim_zones_features))
+  # tests for integer fields
   expect_equal(x$number_of_features(), terra::nlyr(sim_zones_features[[1]]))
   expect_equal(x$number_of_planning_units(), nrow(sim_zones_pu_polygons) - 1)
   expect_equal(
@@ -307,6 +420,7 @@ test_that("x = Spatial, features = ZonesRaster", {
     colnames(x$planning_unit_costs()),
     zone_names(sim_zones_features)
   )
+  expect_error(x$total_unit_ids())
   # tests for feature_abundances_in_planning_units field
   expect_equal(
     x$feature_abundances_in_planning_units(),
@@ -347,6 +461,27 @@ test_that("x = Spatial, features = ZonesRaster", {
     rownames(x$feature_abundances_in_total_units()),
     feature_names(sim_zones_features)
   )
+  # tests for feature_abundances_km2_in_total_units field
+  expect_equal(
+    x$feature_abundances_km2_in_total_units(),
+    sapply(
+      lapply(
+        sim_zones_features, terra::extract, terra::vect(sim_zones_pu_polygons),
+        "sum", na.rm = TRUE, ID = FALSE
+      ),
+      colSums,
+      na.rm = TRUE
+    ) * unit_factor,
+    ignore_attr = TRUE
+  )
+  expect_equal(
+    colnames(x$feature_abundances_km2_in_total_units()),
+    zone_names(sim_zones_features)
+  )
+  expect_equal(
+    rownames(x$feature_abundances_km2_in_total_units()),
+    feature_names(sim_zones_features)
+  )
   # tests for rij_matrix field
   expect_equal(
     x$data$rij_matrix,
@@ -367,6 +502,11 @@ test_that("x = Spatial, features = ZonesRaster", {
     ),
     ignore_attr = TRUE
   )
+  # test for converting total unit ids to indices
+  expect_equal(
+    x$convert_total_unit_ids_to_indices(c(seq_len(3), 1e5, 4)),
+    c(seq_len(3), 1e5, 4)
+  )
   # test that calling targets before they have been initialized throws error
   expect_error(x$feature_targets())
 })
@@ -374,26 +514,38 @@ test_that("x = Spatial, features = ZonesRaster", {
 test_that("x = Spatial, features = character", {
   # import data
   sim_pu_polygons <- get_sim_pu_polygons()
+  # define spatial properties
+  suppressWarnings(
+    sf::st_crs(sim_pu_polygons) <- sf::st_crs(3857)
+  )
+  unit_factor <- as_km2(1, "ha")
   # update data
   sim_pu_polygons$cost[2] <- NA
   sim_pu_polygons$spp1 <- runif(nrow(sim_pu_polygons))
   sim_pu_polygons$spp2 <- c(NA, rpois(nrow(sim_pu_polygons) - 1, 5))
   # create problem
   expect_warning(
-    x <- problem(sf::as_Spatial(sim_pu_polygons), c("spp1", "spp2"), "cost"),
+    x <- problem(
+      sf::as_Spatial(sim_pu_polygons), c("spp1", "spp2"), "cost", "ha"
+    ),
     "deprecated"
   )
   # verify that object can be printed
   suppressMessages(print(x))
   suppressMessages(x)
-  # tests for integer fields
+  # test for logical fields
+  expect_true(x$is_ids_equivalent_to_indices())
+  # tests for character fields
+  expect_equal(x$planning_unit_class(), "SpatialPolygonsDataFrame")
   expect_equal(x$feature_names(), c("spp1", "spp2"))
   expect_equal(x$zone_names(), "cost")
+  # tests for integer fields
   expect_equal(x$number_of_features(), 2)
   expect_equal(x$number_of_zones(), 1)
   expect_equal(x$number_of_planning_units(), nrow(sim_pu_polygons) - 1)
   expect_equal(x$planning_unit_indices(), c(1, seq(3, nrow(sim_pu_polygons))))
   expect_equal(x$number_of_total_units(), nrow(sim_pu_polygons))
+  expect_error(x$total_unit_ids())
   # tests for planning_unit_costs field
   expect_equal(
     x$planning_unit_costs(),
@@ -441,6 +593,26 @@ test_that("x = Spatial, features = character", {
     rownames(x$feature_abundances_in_total_units()),
     c("spp1", "spp2")
   )
+  # tests for feature_abundances_km2_in_total_units field
+  expect_equal(
+    x$feature_abundances_km2_in_total_units(),
+    matrix(
+      colSums(
+        sf::st_drop_geometry(sim_pu_polygons)[, c("spp1", "spp2")],
+        na.rm = TRUE
+      ),
+      ncol = 1
+    ) * unit_factor,
+    ignore_attr = TRUE
+  )
+  expect_equal(
+    colnames(x$feature_abundances_km2_in_total_units()),
+    "cost"
+  )
+  expect_equal(
+    rownames(x$feature_abundances_km2_in_total_units()),
+    c("spp1", "spp2")
+  )
   # tests for rij_matrix field
   rij <- Matrix::sparseMatrix(
     i = c(
@@ -461,6 +633,11 @@ test_that("x = Spatial, features = character", {
   expect_true(all(x$data$rij_matrix[[1]] == rij[[1]]))
   expect_equal(names(x$data$rij_matrix), "cost")
   expect_equal(rownames(x$data$rij_matrix[[1]]),  c("spp1", "spp2"))
+  # test for converting total unit ids to indices
+  expect_equal(
+    x$convert_total_unit_ids_to_indices(c(seq_len(3), 1e5, 4)),
+    c(seq_len(3), 1e5, 4)
+  )
   # test that calling targets before they have been initialized throws error
   expect_error(x$feature_targets())
 })
@@ -468,6 +645,11 @@ test_that("x = Spatial, features = character", {
 test_that("x = Spatial, features = ZonesCharacter", {
   # import data
   sim_zones_pu_polygons <- get_sim_zones_pu_polygons()
+  # define spatial properties
+  suppressWarnings(
+    sf::st_crs(sim_zones_pu_polygons) <- sf::st_crs("epsg:3857")
+  )
+  unit_factor <- as_km2(1, "ha")
   # update data
   sim_zones_pu_polygons$cost_1[2] <- NA
   sim_zones_pu_polygons[3, c("cost_1", "cost_2")] <- NA
@@ -486,16 +668,21 @@ test_that("x = Spatial, features = ZonesCharacter", {
         zone_names = c("z1", "z2"),
         feature_names = c("spp1", "spp2")
       ),
-      c("cost_1", "cost_2")
+      c("cost_1", "cost_2"),
+      "ha"
     ),
     "deprecated"
   )
   # verify that object can be printed
   suppressMessages(print(x))
   suppressMessages(x)
-  # tests for integer fields
+  # test for logical fields
+  expect_true(x$is_ids_equivalent_to_indices())
+  # tests for character fields
+  expect_equal(x$planning_unit_class(), "SpatialPolygonsDataFrame")
   expect_equal(x$feature_names(), c("spp1", "spp2"))
   expect_equal(x$zone_names(), c("z1", "z2"))
+  # tests for integer fields
   expect_equal(x$number_of_features(), 2)
   expect_equal(x$number_of_zones(), 2)
   expect_equal(x$number_of_planning_units(), nrow(sim_zones_pu_polygons) - 1)
@@ -504,6 +691,7 @@ test_that("x = Spatial, features = ZonesCharacter", {
     c(c(1, 2), seq(4, nrow(sim_zones_pu_polygons)))
   )
   expect_equal(x$number_of_total_units(), nrow(sim_zones_pu_polygons))
+  expect_error(x$total_unit_ids())
   # tests for planning_unit_costs field
   expect_equal(
     x$planning_unit_costs(),
@@ -570,6 +758,28 @@ test_that("x = Spatial, features = ZonesCharacter", {
     rownames(x$feature_abundances_in_total_units()),
     c("spp1", "spp2")
   )
+  # tests for feature_abundances_km2_in_total_units field
+  expect_equal(
+    x$feature_abundances_km2_in_total_units(),
+    matrix(
+      colSums(
+        sf::st_drop_geometry(sim_zones_pu_polygons)[,
+          c("spp1_1", "spp2_1", "spp1_2", "spp2_2")
+        ],
+        na.rm = TRUE
+      ),
+      ncol = 2
+    ) * unit_factor,
+    ignore_attr = TRUE
+  )
+  expect_equal(
+    colnames(x$feature_abundances_km2_in_total_units()),
+    c("z1", "z2")
+  )
+  expect_equal(
+    rownames(x$feature_abundances_km2_in_total_units()),
+    c("spp1", "spp2")
+  )
   # tests for rij_matrix field
   r1 <- Matrix::sparseMatrix(
     i = c(
@@ -603,6 +813,11 @@ test_that("x = Spatial, features = ZonesCharacter", {
   expect_equal(names(x$data$rij_matrix), c("z1", "z2"))
   expect_true(all(x$data$rij_matrix[[1]] == rij[[1]]))
   expect_true(all(x$data$rij_matrix[[2]] == rij[[2]]))
+  # test for converting total unit ids to indices
+  expect_equal(
+    x$convert_total_unit_ids_to_indices(c(seq_len(3), 1e5, 4)),
+    c(seq_len(3), 1e5, 4)
+  )
   # test that calling targets before they have been initialized throws error
   expect_error(x$feature_targets())
 })
@@ -621,13 +836,18 @@ test_that("x = sf, features = RasterStack", {
   # verify that object can be printed
   suppressMessages(print(x))
   suppressMessages(x)
-  # tests for integer fields
+  # test for logical fields
+  expect_true(x$is_ids_equivalent_to_indices())
+  # tests for character fields
+  expect_equal(x$planning_unit_class(), "sf")
   expect_equal(x$feature_names(), names(sim_features))
   expect_equal(x$zone_names(), "cost")
+  # tests for integer fields
   expect_equal(x$number_of_features(), terra::nlyr(sim_features))
   expect_equal(x$number_of_planning_units(), sum(!is.na(sim_pu_polygons$cost)))
   expect_equal(x$planning_unit_indices(), which(!is.na(sim_pu_polygons$cost)))
   expect_equal(x$number_of_total_units(), nrow(sim_pu_polygons))
+  expect_error(x$total_unit_ids())
   # tests for planning_unit_costs field
   expect_equal(
     x$planning_unit_costs(),
@@ -682,6 +902,11 @@ test_that("x = sf, features = RasterStack", {
   )
   expect_equal(names(x$data$rij_matrix), "cost")
   expect_equal(rownames(x$data$rij_matrix[[1]]), names(sim_features))
+  # test for converting total unit ids to indices
+  expect_equal(
+    x$convert_total_unit_ids_to_indices(c(seq_len(3), 1e5, 4)),
+    c(seq_len(3), 1e5, 4)
+  )
   # test that calling targets before they have been initialized throws error
   expect_error(x$feature_targets())
 })
@@ -703,9 +928,13 @@ test_that("x = sf, features = ZonesRaster", {
   # verify that object can be printed
   suppressMessages(print(x))
   suppressMessages(x)
-  # tests for integer fields
+  # test for logical fields
+  expect_true(x$is_ids_equivalent_to_indices())
+  # tests for character fields
+  expect_equal(x$planning_unit_class(), "sf")
   expect_equal(x$feature_names(), feature_names(sim_zones_features))
   expect_equal(x$zone_names(), zone_names(sim_zones_features))
+  # tests for integer fields
   expect_equal(x$number_of_features(), terra::nlyr(sim_zones_features[[1]]))
   expect_equal(x$number_of_planning_units(), nrow(sim_zones_pu_polygons) - 1)
   expect_equal(
@@ -713,6 +942,7 @@ test_that("x = sf, features = ZonesRaster", {
     c(seq_len(4), seq(6, nrow(sim_zones_pu_polygons)))
   )
   expect_equal(x$number_of_total_units(), nrow(sim_zones_pu_polygons))
+  expect_error(x$total_unit_ids())
   # tests for planning_unit_costs field
   expect_equal(
     x$planning_unit_costs(),
@@ -791,6 +1021,11 @@ test_that("x = sf, features = ZonesRaster", {
     ),
     ignore_attr = TRUE
   )
+  # test for converting total unit ids to indices
+  expect_equal(
+    x$convert_total_unit_ids_to_indices(c(seq_len(3), 1e5, 4)),
+    c(seq_len(3), 1e5, 4)
+  )
   # test that calling targets before they have been initialized throws error
   expect_error(x$feature_targets())
 })
