@@ -1,5 +1,7 @@
 test_that("x = numeric", {
-  expect_inherits(repr(c(1, 2, 3)), "character")
+  expect_inherits(repr(1), "character")
+  expect_inherits(repr(1:2), "character")
+  expect_inherits(repr(1:3), "character")
 })
 
 test_that("x = logical", {
@@ -71,10 +73,18 @@ test_that("x = Matrix", {
   expect_inherits(repr(Matrix::Matrix(c(1, 2, 3))), "character")
 })
 
+test_that("x = data.frame", {
+  expect_inherits(repr(data.frame(1)), "character")
+})
+
 test_that("x = list", {
   expect_inherits(repr(list(1)), "character")
 })
 
+test_that("x = function", {
+  expect_inherits(repr(mean), "character")
+})
+
 test_that("x = NULL", {
   expect_inherits(repr(NULL), "character")
 })
