@@ -18,12 +18,15 @@ test_that("works", {
   gc1 <- sf::st_geometrycollection(list(p1, ls1, pl1, mp1))
   # create sf object from all possible geometry classes
   pts_geom <- sf::st_as_sf(sf::st_sfc(list(p1)))
+  mp_geom <- sf::st_as_sf(sf::st_sfc(list(mp1)))
   lns_geom <- sf::st_as_sf(sf::st_sfc(list(ls1)))
   ply_geom <- sf::st_as_sf(sf::st_sfc(list(pl1)))
   mply_geom <- sf::st_as_sf(sf::st_sfc(list(mpl1)))
-  all_geom  <- sf::st_as_sf(sf::st_sfc(list(p1, mp1, ls1, ml1, pl1, mpl1, gc1)))
+  all_geom  <-
+    sf::st_as_sf(sf::st_sfc(list(p1, mp1, ls1, ml1, pl1, mpl1, gc1)))
   # tests
   expect_equal(st_geometry_classes(pts_geom), "POINT")
+  expect_equal(st_geometry_classes(mp_geom), "MULTIPOINT")
   expect_equal(st_geometry_classes(lns_geom), "LINESTRING")
   expect_equal(st_geometry_classes(ply_geom), "POLYGON")
   expect_equal(st_geometry_classes(mply_geom), "MULTIPOLYGON")
